﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Management;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows.Forms;

namespace LogonShell
{
    public partial class LogOnForm : Form
	{
		#region P/Invoke
		public enum WTS_CONNECTSTATE_CLASS
		{
			WTSActive,
			WTSConnected,
			WTSConnectQuery,
			WTSShadow,
			WTSDisconnected,
			WTSIdle,
			WTSListen,
			WTSReset,
			WTSDown,
			WTSInit
		}

		public enum WTS_INFO_CLASS
		{
			WTSInitialProgram,
			WTSApplicationName,
			WTSWorkingDirectory,
			WTSOEMId,
			WTSSessionId,
			WTSUserName,
			WTSWinStationName,
			WTSDomainName,
			WTSConnectState,
			WTSClientBuildNumber,
			WTSClientName,
			WTSClientDirectory,
			WTSClientProductId,
			WTSClientHardwareId,
			WTSClientAddress,
			WTSClientDisplay,
			WTSClientProtocolType,
			WTSIdleTime,
			WTSLogonTime,
			WTSIncomingBytes,
			WTSOutgoingBytes,
			WTSIncomingFrames,
			WTSOutgoingFrames,
			WTSClientInfo,
			WTSSessionInfo,
			WTSSessionInfoEx,
			WTSConfigInfo,
			WTSValidationInfo,
			WTSSessionAddressV4,
			WTSIsRemoteSession
		}

		[StructLayout(LayoutKind.Sequential)]
		private struct WTS_SESSION_INFO
		{
			public Int32 SessionID;

			[MarshalAs(UnmanagedType.LPStr)]
			public String pWinStationName;

			public WTS_CONNECTSTATE_CLASS State;
		}

		[DllImport("wtsapi32.dll", SetLastError = true)]
		static extern IntPtr WTSOpenServer(string pServerName);
		[DllImport("wtsapi32.dll", SetLastError = true)]
		static extern int WTSEnumerateSessions(IntPtr hServer, int Reserved, int Version, ref IntPtr ppSessionInfo, ref int pCount);
		[DllImport("wtsapi32.dll", SetLastError = true)]
		static extern bool WTSQuerySessionInformationW(IntPtr hServer, int sessionId, WTS_INFO_CLASS wtsInfoClass, out IntPtr ppBuffer, out int pBytesReturned);
		[DllImport("wtsapi32.dll", SetLastError = true)]
		static extern void WTSFreeMemory(IntPtr value);
		[DllImport("wtsapi32.dll", SetLastError = true)]
		static extern int WTSConnectSession(int LogonId, int TargetLogonId, string pPassword, bool bWait);
		[DllImport("kernel32.dll", SetLastError = true)]
		static extern int WTSGetActiveConsoleSessionId();
		[DllImport("uxtheme.dll", SetLastError = true)]
		static extern int SetWindowTheme(IntPtr window, string app, string theme);
		[DllImport("user32.dll", SetLastError = true)]
		static extern bool ExitWindowsEx(uint uFlags, uint dwReason); [DllImport("advapi32.dll", SetLastError = true, BestFitMapping = false, ThrowOnUnmappableChar = true)]
		[return: MarshalAs(UnmanagedType.Bool)]
		static extern bool LogonUser([MarshalAs(UnmanagedType.LPStr)] string pszUserName, [MarshalAs(UnmanagedType.LPStr)] string pszDomain, [MarshalAs(UnmanagedType.LPStr)] string pszPassword, int dwLogonType, int dwLogonProvider, ref IntPtr phToken);
		[DllImport("user32.dll", SetLastError = true)]
		static extern bool PaintDesktop(IntPtr hdc);

		readonly IntPtr WTS_CURRENT_SERVER_HANDLE = IntPtr.Zero;

        [DllImport("User32.dll")]
        static extern int SetForegroundWindow(IntPtr point);
		#endregion

		bool optionsShown = false;
		private GifImage gifImage = null;
        Control UsernameBox;
        SystemMenuManager menuManager;
		List<string> UsernameList = new List<string>();
		List<string> UserSIDList = new List<string>();
        public LogOnForm(string[] args)
		{
			InitializeComponent();
			Load += Form1_Load;
			gifImage = new GifImage(Settings.Read(19));
			int height;
			if (Int32.TryParse(Settings.Read(25), out height))
				pictureBoxLoading.Height = height;
			else
				pictureBoxLoading.Height = Image.FromFile(Settings.Read(19)).Height;
			pictureBoxLoading.Image = gifImage.GetNextFrame();
			pictureBoxBanner.Image = Image.FromFile(Settings.Read(3));
			if (Boolean.Parse(Settings.Read(17)))
				buttonCancel.Enabled = true;
			if (Settings.Read(18).ToLower() == "enabled")
				ControlBox = true;
			else if (Settings.Read(18).ToLower() == "disabled")
			{
				ControlBox = true;
                this.menuManager = new SystemMenuManager(this, SystemMenuManager.MenuItemState.Greyed);
            }
            LoadLocalisation();
			LoadTheme();
			if (args.Length > 0 && args[0] == "-startup" && AutologinEnabled())
            {
                ShowLoading();
				backgroundWorker1.RunWorkerAsync();
			}
			else
			{
				ShowKeySeqDialog();
				if (Settings.Read(24).ToLower() == "always" && (Settings.Read(16).ToLower() == "last" || Settings.Read(16).ToLower() == "listlast"))
					GetUserPasswordHint();
			}
		}

		#region Form Events
		private void Form1_Load(object sender, EventArgs e)
		{
			if (Settings.Read(28).ToLower() == "classic")
				SetWindowTheme(Handle, " ", " ");
		}

		private void Form1_Shown(object sender, EventArgs e)
        {
            Location = new Point(Location.X, 166);
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            Application.Exit();
        }
		private void Form1_KeyDown(object sender, KeyEventArgs e)
		{
			var keySeqOption = Settings.Read(14).ToLower();
			switch (keySeqOption)
			{
				case "win":
					if (e.KeyCode == Keys.LWin || e.KeyCode == Keys.RWin)
					{
						labelKeySequence.Visible = false;
						pictureBoxKeySequence.Visible = false;
						ShowLogin();
						this.KeyDown -= this.Form1_KeyDown;
					}
					break;
				case "ctrlaltwin":
					if (e.Control && e.Alt && (e.KeyCode == Keys.LWin || e.KeyCode == Keys.RWin))
					{
						labelKeySequence.Visible = false;
						pictureBoxKeySequence.Visible = false;
						ShowLogin();
						this.KeyDown -= this.Form1_KeyDown;
					}
					break;
				case "ctrlaltesc":
					if (e.Control && e.Alt && e.KeyCode == Keys.Escape)
					{
						labelKeySequence.Visible = false;
						pictureBoxKeySequence.Visible = false;
						ShowLogin();
						this.KeyDown -= this.Form1_KeyDown;
					}
					break;
				default:
					MessageBox.Show("Configuration Error: unknown key sequence " + keySeqOption);
					break;
			}
		}
		#endregion

		#region Control Events
		private void textBox_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Enter)
			{
				buttonOK.PerformClick();
			}
		}

		private void buttonOK_Click(object sender, EventArgs e)
		{
            IntPtr user = new IntPtr();
            bool succes = LogonUser(UsernameBox.Text, Environment.MachineName, textBoxPassword.Text, 3, 0, ref user);
            if (!succes)
            {
                if (Marshal.GetLastWin32Error() == 1326)
                {
                    new MessageBoxForm(Settings.Read(21, true), Settings.Read(22, true), 1).ShowDialog();
                    if (Settings.Read(24).ToLower() == "tryfirst")
                        GetUserPasswordHint();
					return;
				}
                else if (Marshal.GetLastWin32Error() != 1327)
				{
					new MessageBoxForm(Settings.Read(23, true), Marshal.GetExceptionForHR(Marshal.GetHRForLastWin32Error()).Message, 0).ShowDialog();
					return;
				}
            }

            ShowLoading();
            try
            {
                Logon.Login(UsernameBox.Text, textBoxPassword.Text, String.Compare(GetLastUserLoggedOn(), UsernameBox.Text, CultureInfo.CurrentCulture, CompareOptions.IgnoreCase | CompareOptions.IgnoreSymbols) == 0);
            }
            catch (Exception ex)
            {
                new MessageBoxForm(Settings.Read(23, true), ex.Message, 0).ShowDialog();
				Application.Exit();
            }
            backgroundWorker1.RunWorkerAsync();
        }

		private void buttonCancel_Click(object sender, EventArgs e)
		{
			Application.Exit();
		}

		private void buttonOptions_Click(object sender, EventArgs e)
		{
			optionsShown = !optionsShown;
			buttonShutdown.Visible = optionsShown;
			if (optionsShown)
			{
				buttonOptions.Text = Settings.Read(8, true) + " < <";
				buttonCancel.Location = buttonOK.Location;
				buttonOK.Location = buttonDummy.Location;
			}
			else
			{
				buttonOptions.Text = Settings.Read(8, true) + " > >";
				buttonOK.Location = buttonCancel.Location;
				buttonCancel.Location = buttonShutdown.Location;
			}
		}

		private void buttonShutdown_Click(object sender, EventArgs e)
		{
			var ShutdownForm = new ShutDownForm();
			var result = ShutdownForm.ShowDialog();
			if (result == DialogResult.OK)
				ShowLoading(true);
			ShutdownForm.Dispose();
        }

        private void ComboBoxUsername_SelectedIndexChanged(object sender, EventArgs e)
        {
            GetUserPasswordHint();
        }
        #endregion

        #region Get User Accounts
        private void PopulateUserList(ComboBox userComboBox)
		{
			userComboBox.Items.Clear();
			SelectQuery query = new SelectQuery("SELECT * FROM Win32_UserAccount WHERE Disabled=False");
			ManagementObjectSearcher searcher = new ManagementObjectSearcher(query);
			foreach (ManagementObject envVar in searcher.Get())
			{
				UsernameList.Add((string)envVar["Name"]);
				UserSIDList.Add((string)envVar["SID"]);
            }
			userComboBox.Items.AddRange(UsernameList.ToArray());
		}
		private string GetLastUserLoggedOn()
		{
			string location = @"SOFTWARE\Microsoft\Windows\CurrentVersion\Authentication\LogonUI";
			var registryView = Environment.Is64BitOperatingSystem ? RegistryView.Registry64 : RegistryView.Registry32;
			using (var hive = RegistryKey.OpenRemoteBaseKey(RegistryHive.LocalMachine, System.Environment.MachineName, registryView))
			{
				using (var key = hive.OpenSubKey(location))
				{
					var item = key.GetValue("LastLoggedOnUser");
					string itemValue = item == null ? "No Logon Found" : item.ToString().Remove(0, 2); ;
					return itemValue;
				}
			}
		}
		private void GetUserPasswordHint()
		{
			try
			{
				string sid = UserSIDList.ElementAt(UsernameList.IndexOf(UsernameBox.Text));
				if (sid != null)
				{
					var rid = sid.Split('-').Last();
					var hexValue = int.Parse(rid).ToString("X").PadLeft(8, '0');
					RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SAM\SAM\Domains\Account\Users\" + hexValue);
					byte[] KeyValue = (byte[])key.GetValue("UserPasswordHint");
					labelHint.Text = System.Text.Encoding.UTF8.GetString(KeyValue, 0, KeyValue.Length).Replace("\0", "");
					labelHint.Visible = true;
				}
			}
			catch (Exception)
			{
				// new MessageBoxForm(Settings.Read(23, true), e.Message, 0).ShowDialog(); do nothing
			}
		}
		#endregion

		#region Miscellaneous

		private void ShowKeySeqDialog()
		{
			var keySeqOption = Settings.Read(14).ToLower();
			if (keySeqOption != "false" && (keySeqOption == "win" || keySeqOption == "ctrlaltwin" || keySeqOption == "ctrlaltesc"))
			{
				HideLogin();
				labelKeySequence.Visible = true;
				pictureBoxKeySequence.Image = Image.FromFile(Settings.Read(15));
				pictureBoxKeySequence.Visible = true;
				this.KeyDown += new System.Windows.Forms.KeyEventHandler(this.Form1_KeyDown);
			}
			else if (keySeqOption.ToLower() == "false")
			{
				LoadUsernameBox();
				Normalize();
			}
			else
				MessageBox.Show("Configuration Error: unknown key sequence " + keySeqOption);
		}

		private void HideLogin()
		{
			textBoxPassword.Visible = false;
			comboBoxUsername.Visible = false;
			textBoxUsername.Visible = false;
			var labels = this.Controls.OfType<Label>();
			foreach (var lbl in labels)
				lbl.Visible = false;
			var buttons = this.Controls.OfType<Button>();
			foreach (var btn in buttons)
				btn.Visible = false;
			this.ClientSize = new System.Drawing.Size(409, 132);
		}

		private void ShowLogin()
		{
			this.ClientSize = new System.Drawing.Size(409, 205);
			textBoxPassword.Visible = true;
			labelPassword.Visible = true;
			labelUsername.Visible = true;
			buttonCancel.Visible = true;
			buttonOK.Visible = true;
			buttonOptions.Visible = true;
			//System.Threading.Thread.Sleep(400);
			MouseOperations.SetCursorPosition(Location.X + 350, Location.Y + 128);
			MouseOperations.MouseEvent(MouseOperations.MouseEventFlags.LeftDown);
			MouseOperations.MouseEvent(MouseOperations.MouseEventFlags.LeftUp);
			LoadUsernameBox();
			UsernameBox.Visible = true;
		}

		private void ShowLoading(bool isShutdown = false)
		{
			HideLogin();
			this.Text = Settings.Read(10, true);
			int fps;
			if (Int32.TryParse(Settings.Read(20), out fps))
			{
				timer1.Interval = 1000 / fps;
				timer1.Start();
			}
			else
				pictureBoxLoading.Image = Image.FromFile(Settings.Read(19));
			if (isShutdown)
				labelShutdown.Visible = true;
			else
				labelLoading.Visible = true;
			pictureBoxBanner.Image = Image.FromFile(Settings.Read(31));
		}

		private void LoadLocalisation()
		{
			this.Text = Settings.Read(5, true);
			buttonOK.Text = Settings.Read(1, true);
			buttonCancel.Text = Settings.Read(2, true);
			buttonOptions.Text = Settings.Read(8, true) + " > >";
			buttonShutdown.Text = Settings.Read(9, true);
			labelPassword.Text = Settings.Read(6, true);
			labelUsername.Text = Settings.Read(7, true);
			labelLoading.Text = Settings.Read(11, true);
            labelShutdown.Text = Settings.Read(14, true);
		}

		private void Normalize()
        {
			if (labelUsername.Width > 70 || labelPassword.Width > 70)
            {
				int newWidth = (labelUsername.Width > labelPassword.Width)? labelUsername.Width : labelPassword.Width;
				newWidth = (newWidth > 142) ? 142 : newWidth;
				UsernameBox.Left = newWidth + 16;
				textBoxPassword.Left = newWidth + 16;
				labelHint.Width = 316;
			}
        }

		private void LoadTheme()
		{
			var labels = this.Controls.OfType<Label>();
			var buttons = this.Controls.OfType<Button>();
			if (Settings.Read(27).ToLower() != "default")
			{
				this.BackColor = ColorTranslator.FromHtml(Settings.Read(27));
				foreach (var lbl in labels)
					lbl.BackColor = ColorTranslator.FromHtml(Settings.Read(27));
                foreach (var btn in buttons)
                    btn.BackColor = SystemColors.Control;
            }
			if (Settings.Read(26).ToLower() != "default")
			{
				foreach (var lbl in labels)
					lbl.ForeColor = ColorTranslator.FromHtml(Settings.Read(26));
            }
		}

		private void LoadUsernameBox()
		{
			var usernameOption = Settings.Read(16).ToLower();
			switch (usernameOption)
			{
				case "simple":
					textBoxUsername.Visible = true;
					comboBoxUsername.Visible = false;
					this.ActiveControl = textBoxUsername;
					textBoxUsername.Focus();
					textBoxUsername.KeyDown += textBox_KeyDown;
					UsernameBox = textBoxUsername;
					break;
				case "last":
					textBoxUsername.Visible = true;
					comboBoxUsername.Visible = false;
					textBoxUsername.Text = GetLastUserLoggedOn();
					textBoxUsername.KeyDown += textBox_KeyDown;
					UsernameBox = textBoxUsername;
					this.ActiveControl = textBoxPassword;
					textBoxPassword.Focus();
					break;
				case "list":
					textBoxUsername.Visible = false;
					comboBoxUsername.Visible = true;
					PopulateUserList(comboBoxUsername);
					this.ActiveControl = comboBoxUsername;
                    comboBoxUsername.Focus();
					if (Settings.Read(24).ToLower() == "always")
                        comboBoxUsername.SelectedIndexChanged += ComboBoxUsername_SelectedIndexChanged;
					comboBoxUsername.KeyDown += textBox_KeyDown;
					UsernameBox = comboBoxUsername;
					break;
				case "listlast":
					textBoxUsername.Visible = false;
					comboBoxUsername.Visible = true;
					PopulateUserList(comboBoxUsername);
					comboBoxUsername.Text = GetLastUserLoggedOn();
					if (Settings.Read(24).ToLower() == "always")
						comboBoxUsername.SelectedIndexChanged += ComboBoxUsername_SelectedIndexChanged;
					comboBoxUsername.KeyDown += textBox_KeyDown;
					UsernameBox = comboBoxUsername;
					this.ActiveControl = textBoxPassword;
					textBoxPassword.Focus();
					break;
				default:
					MessageBox.Show("Configuration Error: unknown login type " + usernameOption);
					break;
			}
		}

		private void backgroundWorker1_DoWork(object sender, System.ComponentModel.DoWorkEventArgs e)
		{
			Process p = Process.GetProcessesByName("LogonUI").FirstOrDefault();
			do
			{
				p = Process.GetProcessesByName("LogonUI").FirstOrDefault();
				Thread.Sleep(100);
			} while (p != null);
			Application.Exit();
		}

        private bool AutologinEnabled()
        {
            IntPtr phToken = new IntPtr();

            bool loggedIn = LogonUser(GetLastUserLoggedOn(), Environment.MachineName, "", 3, 0, ref phToken);

            int error = Marshal.GetLastWin32Error();

            bool autologon = false;
			bool displayLastUsername = true;

            RegistryKey key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Microsoft\Windows NT\CurrentVersion\Winlogon");
            if (key.GetValue("AutoAdminLogon") != null)
                autologon = (key.GetValue("AutoAdminLogon").ToString()).Contains("1");
            key = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Microsoft\Windows\CurrentVersion\Policies\System");
            if (key.GetValue("DontDisplayLastUserName") != null)
                displayLastUsername = (key.GetValue("DontDisplayLastUserName").ToString()).Contains("0");

            // 1327 = empty password
            if ((loggedIn || error == 1327 || autologon) && displayLastUsername)
                return true;
            else
                return false;
		}
		private void timer1_Tick(object sender, EventArgs e)
		{
			pictureBoxLoading.Image = gifImage.GetNextFrame();
		}

		#endregion

	}
}
